/******************************************************
STM8S003K3 16MHz, IAR1.4+STM8S_StdPeriph_Lib_V2.1.0
******************************************************/
#include "stm8s.h"
#include "iostm003.c"
#include "bitset.c"

#define A       PD_ODR_ODR7
#define B       PD_ODR_ODR6
#define C       PD_ODR_ODR5
#define D       PD_ODR_ODR4
#define E       PD_ODR_ODR3
#define F       PD_ODR_ODR2
#define G       PD_ODR_ODR1
#define DP      PC_ODR_ODR7
#define N0      PC_ODR_ODR6
#define N1      PC_ODR_ODR5
#define N2      PC_ODR_ODR4
#define N3      PC_ODR_ODR3

#define key	    PB_IDR_IDR7


#define Rhv_upr 680000 //Ohms
#define Rhv_lwr 13000   //Ohms
#define UHV_max 	((500UL*(Rhv_upr+Rhv_lwr))/Rhv_lwr) //=U_max*100, 266V*100
#define Rbt_upr 22000 //Ohms
#define Rbt_lwr 6800   //Ohms
#define Ubat_max 	((500UL*(Rbt_upr+Rbt_lwr))/Rbt_lwr) //=U_max*100, 21.17V*100


enum btn_states {prs, rls, hold} btn=rls;
enum dmodes {normal,started,lobat,ocp,sdn,ovl} d_mode=started;

u8 dbuf[4]={1,2,3,4};
u8 dppos=0;
volatile u16 adc2val=1023, adc3val=0;
u8 a; 
volatile u8 cntfb;
u8 tmr0,tmr1,tmr2=50;

void gpio_cfg()
{
  PA_DDR=PB_DDR=PC_DDR=PD_DDR=PE_DDR=PF_DDR=0xff; //conf as output
  PA_CR1=PB_CR1=PC_CR1=PD_CR1=PE_CR1=PF_CR1=0xff; //push-pull|pullup
  PB_DDR_DDR2=PB_DDR_DDR3=PB_DDR_DDR7=0;//ADC2,3, PB7_IN
  PB_CR1_C13=PB_CR1_C14=0;//pullup off on adc pins
  PD_DDR_DDR0=0; //BKIN
  PB_ODR_ODR1=PB_ODR_ODR2=PC_ODR_ODR1=PC_ODR_ODR2=1;
}
void clk_cfg()
{
  CLK_DeInit();
  CLK->CKDIVR = 0;
  CLK->PCKENR1 = 0x80; //only tim1&adc clk enabled
}
void t1_cfg()
{
  {
  TIM1->ARRH    = 1;
  TIM1->ARRL    = 0xff;
  TIM1->CCR1H=TIM1->CCR2H=1;
  TIM1->CCR1L=TIM1->CCR2L=0;
  TIM1->CR1     = TIM1_COUNTERMODE_UP|TIM1_CR1_ARPE|TIM1_CR1_URS;
  TIM1->CR2     = TIM1_CR2_CCPC;
  TIM1->IER     = TIM1_IER_UIE;
  }

  TIM1->CCER1 = TIM1_CCER1_CC1E|TIM1_CCER1_CC1NE|TIM1_CCER1_CC1P|TIM1_CCER1_CC1NP;
  TIM1->CCER1|= TIM1_CCER1_CC2E|TIM1_CCER1_CC2NE|TIM1_CCER1_CC2P|TIM1_CCER1_CC2NP;

  TIM1->CCMR1 = TIM1_OCMODE_PWM1;
  TIM1->CCMR2 = TIM1_OCMODE_PWM1;

  TIM1->EGR  |= TIM1_EGR_COMG; //update CCER reg with COM event

  TIM1->OISR = 0xff; //iddle state
  TIM1->BKR  = TIM1_BKR_OSSI|TIM1_BKR_BKE;
  TIM1->DTR  = 4; //240ns dt delay (62ns*4)
  TIM1->CR1 |= TIM1_CR1_CEN;
  TIM1->BKR |= TIM1_LOCKLEVEL_3;

}
void adc_cfg()
{
  ADC1->CR1 = ADC1_PRESSEL_FCPU_D18;
  ADC1->CR2 = ADC1_ALIGN_RIGHT;
  ADC1->CSR = 3; //channel sel 2 for HV, 3 for bat

  ADC1->TDRL = 1<<3|1<<2; //smtt trg on adc pins is off
  ADC1->CR1 |= ADC1_CR1_ADON;
}
void iwdg_cfg()
{
IWDG_WriteAccessCmd(IWDG_WriteAccess_Enable);
IWDG_SetPrescaler(IWDG_Prescaler_4);
IWDG_SetReload(0);
IWDG_Enable();
IWDG_ReloadCounter();
}
void convert_data (u16 d)
{
  for (u8 k=0;k<4;k++)
  {
   dbuf[k]=d%10;
   d/=10;
  }
}
void hv_cntrl()
{
 u16 tmpU;

 if(cntfb)
 {
  tmpU = ((u32)UHV_max*adc2val)/(0x3ffUL*100);

	if((tmpU>230)&&(tmpU<240))	{tmr2=250;if(a){a--;}}
	else if((tmpU>200)&&(tmpU<210)){tmr2=250;if(a<100){a++;}}
	else if(tmpU>=240){if(a>5){a-=5;tmr2=250;}else a=0;}
	else if((tmpU<=200)&&(a<100)){if(a<100-5){a+=5;tmr2=250;}else a=100;}
	//else if (tmpU<200){if(a<0xff){a++;}else {a=0xff;}}
	
	//if(!tmr2){d_mode=ovl;} else if(a<100) {d_mode=normal;}

 cntfb=0;
 }
}
u8 bat_ok()
{
 u8 tmp;
 static u8 trg0;
 
 tmp = ((u32)Ubat_max*adc3val)/(0x3ffUL*10);
	 
 if(tmp>108){tmr0=100;}   
 //else if(tmp<100){d_mode=lobat; trg0=1;}
 
 if(tmp>120) 	trg0=0; 
 if(!tmr0)	trg0=1;
 
 if (!trg0)	return 1; else return 0;
}


void display_data ()
{
  static int pc;

  (pc)?(pc--):(pc = 3);
  N0=N1=N2=N3=0; DP=1;

  switch (dbuf[pc])
 {

  case 0:  A=0; B=0; C=0; D=0; E=0; F=0; G=1; break;
  case 1:  A=1; B=0; C=0; D=1; E=1; F=1; G=1; break;
  case 2:  A=0; B=0; C=1; D=0; E=0; F=1; G=0; break;
  case 3:  A=0; B=0; C=0; D=0; E=1; F=1; G=0; break;
  case 4:  A=1; B=0; C=0; D=1; E=1; F=0; G=0; break;
  case 5:  A=0; B=1; C=0; D=0; E=1; F=0; G=0; break;
  case 6:  A=0; B=1; C=0; D=0; E=0; F=0; G=0; break;
  case 7:  A=0; B=0; C=0; D=1; E=1; F=1; G=1; break;
  case 8:  A=0; B=0; C=0; D=0; E=0; F=0; G=0; break;
  case 9:  A=0; B=0; C=0; D=0; E=1; F=0; G=0; break;
  case 10: A=1; B=1; C=1; D=1; E=1; F=1; G=1; break;
  case 11: A=1; B=1; C=0; D=0; E=0; F=1; G=1; break; //u
  case 12: A=1; B=1; C=1; D=0; E=0; F=1; G=0; break; //c
  case 13: A=1; B=1; C=0; D=0; E=0; F=0; G=0; break; //b
  case 14: A=1; B=1; C=1; D=0; E=0; F=0; G=0; break; //t
  case 15: A=1; B=1; C=1; D=0; E=0; F=1; G=1; break; //l
  case 16: A=1; B=1; C=0; D=0; E=0; F=1; G=0; break; //o
  case 17: A=0; B=0; C=0; D=1; E=0; F=0; G=0; break; //A
  case 18: A=1; B=0; C=0; D=1; E=0; F=0; G=0; break; //H
  case 19: A=0; B=1; C=1; D=0; E=0; F=0; G=1; break; //C
  case 20: A=0; B=0; C=1; D=1; E=0; F=0; G=0; break; //P
  case 21: A=0; B=1; C=1; D=0; E=0; F=0; G=0; break; //E
  case 22: A=1; B=0; C=0; D=0; E=0; F=0; G=1; break; //U
  case 23: A=0; B=1; C=1; D=1; E=0; F=0; G=0; break; //F
  case 24: A=1; B=1; C=1; D=0; E=0; F=0; G=1; break; //L
  case 25: A=1; B=1; C=1; D=1; E=0; F=1; G=0; break; //r
  case 26: A=1; B=1; C=1; D=0; E=1; F=1; G=1; break; //_
  case 27: A=1; B=0; C=0; D=0; E=0; F=1; G=0; break; //d
  case 28: A=1; B=1; C=0; D=1; E=0; F=1; G=0; break; //n
  case 29: A=1; B=1; C=0; D=1; E=0; F=0; G=0; break; //h
  case 30: A=0; B=0; C=1; D=1; E=1; F=0; G=1; break; //n(up)	
  default: A=1; B=1; C=1; D=1; E=1; F=1; G=0; break; //-
 }

   switch (pc)
 {
  case 0: N0=1; if(dppos == 0) {DP=0;} break;
  case 1: N1=1; if(dppos == 1) {DP=0;} break;
  case 2: N2=1; if(dppos == 2) {DP=0;} break;
  case 3: if(dbuf[3]){N3=1;} if(dppos == 3) {DP=0;} break;
  default: DP=1; break;
 }
 }
void scan_keys ()
{
#define lp  50
#define lh  250

static u8 count[2];

if (!key) {count[0]++; }
else {count[0]=count[1]=0; btn=rls;}

if ((count[0]>lp)&&(btn!=hold)) {btn=prs; count[0]=0; count[1]++;}

if (count[1]>lh) {btn=hold; count[1]=0;}
}
void scan_adc()
{

#define rcntdef2 600
#define rcntdef3 4
  
  static u16 rcnt2=rcntdef2; 
  static u8  rcnt3=rcntdef3, trg=0;
  static u32 tmp2=0;
  static u16 tmp3=0;
  u8  adch,adcl;

	
	{
		
		if(trg)
		{
			if(rcnt3) //Vbat
			{
				rcnt3--;
				adcl = ADC1->DRL;
				adch = ADC1->DRH;
				tmp3+=(u16)(adcl|(u16)(adch<<8));
			}
			else
			{
				rcnt3=rcntdef3; adc3val=tmp3/(rcntdef3); tmp3=0;
			}
			ADC1->CSR = 2;
			trg=0;
		}
		else  //hv sense
		{
			if(rcnt2)
			{
			 rcnt2--;
			 adcl = ADC1->DRL;
			 adch = ADC1->DRH;
			 tmp2+=(adcl|(u16)(adch<<8));
			}
			else
			{
				rcnt2=rcntdef2;
				adc2val=tmp2/(rcntdef2);tmp2=0;trg=1;ADC1->CSR = 3;
			}
		

		}
 	ADC1->CR1 |= ADC1_CR1_ADON;	
	}
}
void dev_mgmt();

INTERRUPT_HANDLER(TIM1_X3_Handler, 11)
{
#define s_step 312
static u8 c=0,d=1,p=1;
static u8 mst=0xff/4;
u16 tmp_table;

static const u8 s_table [s_step/2]=
{0,2,5,7,10,12,15,18,20,23,25,28,30,33,36,38,41,43,46,48,51,
53,56,58,61,63,66,68,71,73,76,78,81,83,86,88,91,93,95,98,100,103,
105,107,110,112,114,117,119,121,123,126,128,130,132,135,137,139,141,143,145,147,150,
152,154,156,158,160,162,164,166,168,170,172,174,175,177,179,181,183,185,186,188,190,
192,193,195,197,198,200,201,203,205,206,208,209,211,212,214,215,216,218,219,220,222,
223,224,225,227,228,229,230,231,232,233,234,235,236,237,238,239,240,241,242,243,243,
244,245,246,246,247,248,248,249,249,250,251,251,252,252,252,253,253,253,254,254,254,
255,255,255,255,255,255,255,255,255};

tmp_table=((u16)s_table [c]*mst)/100;
if(tmp_table>0xff){tmp_table=0xff;}

if((d)&&(p))
 {
  TIM1->CCR1L=TIM1->CCR2L=(u8)tmp_table;
  if(c<s_step/2-1){c++;} else {d=0; if(tmr0)tmr0--;}
 }
else if((!d)&&(p))
  {
  TIM1->CCR1L=TIM1->CCR2L=(u8)tmp_table;
  if(c){c--;} else {d=1;p=0;TIM1->CCR1H=TIM1->CCR2H=0;if(tmr1)tmr1--;}
  }
else if((d)&&(!p))
  {
  TIM1->CCR1L=TIM1->CCR2L=0xff-(u8)tmp_table;
  if(c<s_step/2-1){c++;} else {d=0; if(tmr2)tmr2--;}
  }
else
  {
  TIM1->CCR1L=TIM1->CCR2L=0xff-(u8)tmp_table;
  if(c){c--;}else {d=1;p=1;TIM1->CCR1H=TIM1->CCR2H=1; cntfb=1;mst=a; }
  }
  scan_adc(); 	 
  IWDG_ReloadCounter();
  TIM1->SR1 = 0;
}

void main()
{
gpio_cfg();
clk_cfg();
t1_cfg();
adc_cfg();
iwdg_cfg();

ADC1->CR1 |= ADC1_CR1_ADON;

asm("rim");

while(1)
{
	dev_mgmt();
}
}


void dev_mgmt()
{
  static u8 trgB;
	
	if(btn==rls){trgB=0;}
	
 switch (d_mode)
 {
  case started:
  {
		display_data (); scan_keys ();  
		if((btn==hold)&&(!trgB)){trgB=1;d_mode=sdn;}
		
		if (!bat_ok())
		{
			TIM1->EGR|=TIM1_EGR_BG;
			if(!tmr1)
			{
				tmr1=25;
				convert_data (((u32)Ubat_max*adc3val)/(0x3ffUL*10)); dppos=1;				
			}
		}
		else {a=0xff/4;d_mode=normal;TIM1->BKR|=TIM1_BKR_MOE;}

	}
  break;

  case normal:
  {
	 	hv_cntrl();
	 	display_data(); scan_keys();  
		if((btn==hold)&&(!trgB)){trgB=1;d_mode=sdn;}
		if(!bat_ok()){d_mode=lobat;}
		if(!(TIM1->BKR&TIM1_BKR_MOE)){d_mode=ocp;}
		if(!tmr1){tmr1=25;convert_data (((u32)Ubat_max*adc3val)/(0x3ffUL*10));dppos=1;}

  }
  break;

  case sdn:
  {
	static u8 trgA=0;
	 
	scan_keys(); display_data();

	if(!trgA){trgA=1;TIM1->EGR|=TIM1_EGR_BG;for(s8 i=3;i>=0;i--){dbuf[i]=26;}dppos=4;}
   if((btn==hold)&&(!trgB)){trgB=1;d_mode=started;}
	if(d_mode!=sdn){trgA=0;}
  }
  break;

  case ocp:
  {
	 static u8 trgA=0;
	 
	 scan_keys(); display_data();
	 
	 if(!trgA){trgA=1;dbuf[0]=0;dbuf[1]=0;dbuf[2]=19;dbuf[3]=20;dppos=4;}//OCP
   if((btn==hold)&&(!trgB)){trgB=1;d_mode=started;}
	 if(d_mode!=ocp){trgA=0;}
	}
  break;

  case lobat:
  {
	 static u8 trgA=0;
	 
	 if(!trgA){trgA=1;TIM1->EGR|=TIM1_EGR_BG;}	
	
	 scan_keys(); display_data();
	 
	 if(!tmr1)
	 {
		tmr1=50;
		convert_data (((u32)Ubat_max*adc3val)/(0x3ffUL*10));dbuf[0]=24;dppos=1;
		if(!dbuf[2]){dbuf[2]=10;}
	 	if(bat_ok()){d_mode=started;}
	 }
    if(((btn==hold)&&(!trgB))||(((u32)Ubat_max*adc3val)/(0x3ffUL*10))<90)
	 {trgB=1;d_mode=sdn;}	 
	 
	 if(d_mode!=lobat){trgA=0;}
  }
  break;
                                
	case ovl:
  {
	 static u8 trgA=0;
	 
		if(!trgA){trgA=1;dbuf[0]=11;dbuf[1]=30;dbuf[2]=11;dbuf[3]=30;dppos=4;}	 
		display_data(); scan_keys(); hv_cntrl();  

		if(((btn==hold)&&(!trgB)))//||(((((u32)UHV_max*adc2val)/(0x3ffUL*100)))<180))			
			{d_mode=sdn; trgB=1;}
		if(!bat_ok()){d_mode=lobat;}
		if(!(TIM1->BKR&TIM1_BKR_MOE)){d_mode=ocp;}
		if(d_mode!=ovl){trgA=0;}
	}
  break;

	default: break;
 }
}

